@@ -1,11 +1,13 @@
 import urllib
 import functools
+import django.forms as forms
+import authentic2.saml.common as saml_common
 
 from django.utils.translation import gettext_noop
-import django.forms as forms
 from django.http import HttpResponseRedirect
+from django.contrib.auth import REDIRECT_FIELD_NAME
 
-import authentic2.saml.common as saml_common
+from saml2_endpoints import view_profile
 
 class AuthSAML2Form(forms.Form):
     def __init__(self, *args, **kwargs):
@@ -35,8 +37,17 @@ class AuthSAML2Frontend(object):
 
     def post(self, request, form, nonce, next):
         provider_id = form.cleaned_data['provider_id']
-        return HttpResponseRedirect('/authsaml2/sso?entity_id=%s' %
-                urllib.quote(provider_id))
+        return HttpResponseRedirect('/authsaml2/sso?entity_id=%s&%s=%s' %
+                (urllib.quote(provider_id),
+                REDIRECT_FIELD_NAME,
+                urllib.quote(next)))
+
+    def get_context(self):
+        '''Specific context variable used by the specific template'''
+        return { 'idp_providers': self.idp_list }
 
     def template(self):
         return 'auth/saml2/login_form.html'
+
+    def profile(self, request, next=''):
+        return view_profile(request, next)
