@@ -9,6 +9,8 @@ urlpatterns = patterns('',
     (r'^finish_federation$', finish_federation),
     (r'^singleSignOnArtifact$', singleSignOnArtifact),
     (r'^singleSignOnPost$', singleSignOnPost),
+    # Receive request from functions
+    (r'^sp_slo/(.*)$', sp_slo),
     # Receive request from user interface
     (r'^logout$', logout),
     # Receive response from Redirect SP initiated
@@ -25,4 +27,6 @@ urlpatterns = patterns('',
     (r'^manageNameIdSOAP$', manageNameIdSOAP),
     # Receive request from Redirect IdP initiated
     (r'^manageNameId$', manageNameId),
+    # Receive request from Redirect IdP initiated
+    (r'^delete_federation$', delete_federation),
 )
