@@ -8,7 +8,6 @@ from django.contrib.auth.models import User
 from django.core.exceptions import ImproperlyConfigured
 from django.utils.importlib import import_module
 
-logging.basicConfig()
 
 def load_backend(path):
     '''Load an IdP backend by its module path'''
@@ -32,9 +31,3 @@ def get_backends(setting_name='IDP_BACKENDS'):
     for backend_path in getattr(settings, setting_name, ()):
         backends.append(load_backend(backend_path))
     return backends
-
-class AdminBackend(object):
-    def service_list(self, request):
-        if request.user.is_staff:
-            return (('/admin', _('Authentic administration')),)
-        return []
