@@ -17,15 +17,17 @@ class SamlBackend(object):
                 protocol = 'idff12'
             else:
                 protocol = 'saml2'
-            uri = '/idp/%s/idp_sso/%s' % (protocol, entity_id)
-            name = liberty_provider.name
-            list.append((uri, name))
+            uri = '/idp/%s/idp_sso/' %protocol
+            name = '%s login' %liberty_provider.name
+            provider_id = entity_id
+            list.append((uri, name, provider_id))
             if models.LibertySession.objects.filter(
                     django_session_key=request.session.session_key,
                     provider_id=entity_id).exists():
-                uri = '/idp/%s/idp_slo/%s?next=/' % (protocol, entity_id)
+                uri = '/idp/%s/idp_slo/' %protocol
                 name = liberty_provider.name + ' logout'
-                list.append((uri, name))
+                provider_id = entity_id
+                list.append((uri, name, provider_id))
         return list
 
     def logout_list(self, request):
