@@ -12,6 +12,7 @@ from django.conf import settings
 from django.http import HttpResponseRedirect, Http404, HttpResponse
 from django.shortcuts import render_to_response
 from django.utils.translation import ugettext as _
+from django.db import transaction
 
 from models import *
 import models
@@ -19,6 +20,8 @@ import saml2utils
 import saml11utils
 import authentic2.utils
 
+from authentic2.authsaml2 import signals
+
 AUTHENTIC_STATUS_CODE_NS = "http://authentic.entrouvert.org/status_code/"
 AUTHENTIC_STATUS_CODE_UNKNOWN_PROVIDER = AUTHENTIC_STATUS_CODE_NS + \
     "UnknownProvider"
@@ -33,6 +36,8 @@ AUTHENTIC_STATUS_CODE_MISSING_DESTINATION = AUTHENTIC_STATUS_CODE_NS + \
 AUTHENTIC_STATUS_CODE_INTERNAL_SERVER_ERROR = AUTHENTIC_STATUS_CODE_NS + \
     "InternalServerError"
 
+logger = logging.getLogger('authentic2.saml')
+
 def get_soap_message(request, on_error_raise = True):
     '''Verify that POST content looks like a SOAP message and returns it'''
     if request.method != 'POST' or \
@@ -215,20 +220,27 @@ def load_federation(request, login, user = None):
     '''Load an identity dump from the database'''
     if not user:
         user = request.user
+    logger.debug('load_federation: user is %s' %user.username)
     try:
         q = LibertyIdentityDump.objects.get(user = user)
-        login.setIdentityFromDump(q.identity_dump.encode('utf8'))
+        logger.debug('load_federation: identity dump found %s' %q.identity_dump.encode('utf8'))
     except ObjectDoesNotExist:
         pass
+    else:
+        login.setIdentityFromDump(q.identity_dump.encode('utf8'))
+        logger.debug('load_federation: set identity from dump done %s' %login.identity.dump())
 
-
-def load_session(request, login, session_key = None):
+def load_session(request, login, session_key = None,
+        kind=LIBERTY_SESSION_DUMP_KIND_IDP):
     '''Load a session dump from the database'''
     if not session_key:
         session_key = request.session.session_key
     try:
-        q = LibertySessionDump.objects.get(django_session_key = session_key)
+        q = LibertySessionDump.objects.get(django_session_key=session_key,
+                kind=kind)
+        logger.debug('load_session: session dump found %s' %q.session_dump.encode('utf8'))
         login.setSessionFromDump(q.session_dump.encode('utf8'))
+        logger.debug('load_session: set session from dump done %s' %login.session.dump())
     except ObjectDoesNotExist:
         pass
 
@@ -244,13 +256,14 @@ def save_federation(request, login, user = None):
             q.identity_dump = None
         q.save()
 
-def save_session(request, login, session_key = None):
+def save_session(request, login, session_key=None,
+        kind=LIBERTY_SESSION_DUMP_KIND_IDP):
     '''Save session dump to database'''
     if not session_key:
         session_key = request.session.session_key
     if login.isSessionDirty:
         q, creation = LibertySessionDump.objects.get_or_create(
-                django_session_key = session_key)
+                django_session_key=session_key, kind=kind)
         if login.session:
             q.session_dump = login.session.dump()
         else:
@@ -269,7 +282,6 @@ def delete_session(request):
         except:
             pass
 
-
 def save_manage(request, manage):
     if not request or not manage:
         raise Exception('Cannot save manage dump')
@@ -282,6 +294,7 @@ def get_manage_dump(request):
     d = LibertyManageDump.objects.filter(django_session_key = request.session.session_key)
     return d
 
+@transaction.commit_on_success
 def retrieve_metadata_and_create(request, provider_id, sp_or_idp):
     if not provider_id.startswith('http'):
         return None
@@ -300,10 +313,10 @@ for entity id %r is not UTF-8' % provider_id)
         return None
     p = LibertyProvider(metadata=metadata)
     try:
-        p.clean()
-    except ValidationError:
+        p.full_clean(exclude=['entity_id','protocol_conformance'])
+    except ValidationError, e:
         logging.error('SAML metadata autoload: retrieved metadata \
-for entity id %r are invalid' % provider_id)
+for entity id %r are invalid, %s' % (provider_id, e.args))
         return None
     except:
         logging.exception('SAML metadata autoload: retrieved metadata validation raised an unknown exception')
@@ -313,10 +326,12 @@ for entity id %r are invalid' % provider_id)
         s = LibertyServiceProvider(liberty_provider=p, enabled=True, ask_user_consent=True)
         s.save()
     elif sp_or_idp == 'idp':
-        raise NotImplementedError()
+        s = LibertyIdentityProvider(liberty_provider=p, enabled=True)
+        s.save()
     return p
 
-def load_provider(request, provider_id, server=None, sp_or_idp='sp'):
+def load_provider(request, provider_id, server=None, sp_or_idp='sp',
+        autoload=False):
     '''Look up a provider in the database, and verify it handles wanted
        role be it sp or idp.
 
@@ -332,7 +347,7 @@ def load_provider(request, provider_id, server=None, sp_or_idp='sp'):
         liberty_provider = LibertyProvider.objects.get(entity_id=provider_id)
     except LibertyProvider.DoesNotExist:
         autoload = getattr(settings, 'SAML_METADATA_AUTOLOAD', 'none')
-        if autoload == 'sp' or autoload == 'both':
+        if autoload and (autoload == 'sp' or autoload == 'both'):
             liberty_provider = retrieve_metadata_and_create(request, provider_id, sp_or_idp)
             if not liberty_provider:
                 return False
@@ -364,7 +379,7 @@ def load_provider(request, provider_id, server=None, sp_or_idp='sp'):
     return liberty_provider
 
 # Federation management
-def add_federation(user, login=None, name_id=None):
+def add_federation(user, login=None, name_id=None, provider_id=None):
     if not name_id:
         if not login:
             return None
@@ -382,6 +397,8 @@ def add_federation(user, login=None, name_id=None):
     fed.name_id_qualifier = qualifier
     fed.name_id_sp_name_qualifier = name_id.sPNameQualifier
     fed.name_id_format = name_id.format
+    if provider_id:
+        fed.idp_id = provider_id
     fed.save()
     return fed
 
@@ -391,7 +408,20 @@ def lookup_federation_by_name_identifier(name_id=None, profile=None):
     if not name_id:
         name_id = profile.nameIdentifier
     kwargs = models.nameid2kwargs(name_id)
-    return LibertyFederation.objects.get(**kwargs)
+    try:
+        return LibertyFederation.objects.get(**kwargs)
+    except:
+        return None
+
+def lookup_federation_by_name_id_and_provider_id(name_id, provider_id):
+    '''Try to find a LibertyFederation object for the given NameID and
+       the provider id.'''
+    kwargs = models.nameid2kwargs(name_id)
+    kwargs['idp_id'] = provider_id
+    try:
+        return LibertyFederation.objects.get(**kwargs)
+    except:
+        return None
 
 # TODO: Does it happen that a user have multiple federation with a same idp? NO
 def lookup_federation_by_user(user, qualifier):
@@ -567,9 +597,9 @@ def send_soap_request(request, profile):
     '''Send the SOAP request hold by the profile'''
     if not profile.msgUrl or not profile.msgBody:
         raise SOAPException('Missing body or url')
-    p = LibertyProvider.objects.get(entity_id=profile.remoteProviderId)
-    return soap_call(profile.msgUrl, profile.msgBody, p.ssl_certificate)
-
+    #p = LibertyProvider.objects.get(entity_id=profile.remoteProviderId)
+    #return soap_call(profile.msgUrl, profile.msgBody, p.ssl_certificate)
+    return soap_call(profile.msgUrl, profile.msgBody, None)
 
 def set_saml2_response_responder_status_code(response, code):
     response.status = lasso.Samlp2Status()
@@ -579,13 +609,17 @@ def set_saml2_response_responder_status_code(response, code):
     response.status.statusCode.statusCode.value = code
 
 __root_refererer_re = re.compile('^(https?://[^/]*/?)')
-def error_page(request, message, back = None):
+
+def error_page(request, message, back = None, logger = None):
     '''View that show a simple error page to the user with a back link.
 
          back - url for the back link, if None, return to root of the referer
                 or the local root.
     '''
-    logging.error('Showing message %r on an error page' % message)
+    if logger:
+        logger.error('Showing message %r on an error page' % message)
+    else:
+        logging.error('Showing message %r on an error page' % message)
     if back is None:
         referer = request.META.get('HTTP_REFERER')
         if referer:
@@ -616,3 +650,56 @@ def iso8601_to_datetime(date_string):
         raise ValueError('Invalid ISO8601 date')
     tm = time.strptime(m.group(1)+'Z', "%Y-%m-%dT%H:%M:%SZ")
     return datetime.datetime.fromtimestamp(time.mktime(tm))
+
+def get_idp_options_policy(provider):
+    try:
+        return IdPOptionsSPPolicy.objects.get(name='All', enabled=True)
+    except IdPOptionsSPPolicy.DoesNotExist:
+        pass
+    if provider.identity_provider.enable_following_idp_options_policy:
+        if provider.identity_provider.idp_options_policy:
+            return provider.identity_provider.idp_options_policy
+    try:
+        return IdPOptionsSPPolicy.objects.get(name='Default', enabled=True)
+    except IdPOptionsSPPolicy.DoesNotExist:
+        pass
+    return None
+
+def get_authorization_policy(provider):
+    try:
+        return AuthorizationSPPolicy.objects.get(name='All', enabled=True)
+    except AuthorizationSPPolicy.DoesNotExist:
+        pass
+    if provider.identity_provider.enable_following_authorization_policy:
+        if provider.identity_provider.authorization_policy:
+            return provider.identity_provider.authorization_policy
+    try:
+        return AuthorizationSPPolicy.objects.get(name='Default', enabled=True)
+    except AuthorizationSPPolicy.DoesNotExist:
+        pass
+    return None
+
+def attributesMatch(attributes, attributes_to_check):
+    attrs = AuthorizationAttributeMapping. \
+        objects.filter(map=attributes_to_check)
+    for attr in attrs:
+        if not attributes.has_key(attr.attribute_name):
+            raise Exception('Attribute %s not provided' %attr.attribute_name)
+        if not attr.attribute_value in attributes[attr.attribute_name]:
+            return False
+    return True
+
+def authz_decision_cb(sender, request=None, attributes={},
+            provider=None, **kwargs):
+    p = get_authorization_policy(provider)
+    dic = {}
+    dic['authz'] = True
+    if p and p.attribute_map:
+        dic['authz'] = attributesMatch(attributes, p.attribute_map)
+        if not dic['authz']:
+            dic['message'] = \
+            _('Your access is denied. At least one attribute does not match.')
+    return dic
+
+signals.authz_decision.connect(authz_decision_cb,
+    dispatch_uid='authz_decision_on_attributes')
