@@ -4,6 +4,7 @@ import xml.etree.ElementTree as etree
 import hashlib
 import binascii
 import base64
+import datetime
 
 import lasso
 from django.db import models
@@ -13,6 +14,7 @@ from django.core.exceptions import ValidationError
 from django.core.files.storage import FileSystemStorage
 from django.utils.translation import ugettext as _
 from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
+from django.utils.importlib import import_module
 
 from fields import *
 
@@ -20,17 +22,6 @@ from fields import *
 ATTRIBUTE_VALUE_FORMATS = (
         (lasso.SAML2_ATTRIBUTE_NAME_FORMAT_URI, 'SAMLv2 URI'),)
 
-class LibertyAttributeMap(models.Model):
-    name = models.CharField(max_length = 40, unique = True)
-    def __unicode__(self):
-        return self.name
-
-class LibertyAttributeMapping(models.Model):
-    source_attribute_name = models.CharField(max_length = 40)
-    attribute_value_format = models.URLField(choices = ATTRIBUTE_VALUE_FORMATS)
-    attribute_name = models.CharField(max_length = 40)
-    map = models.ForeignKey(LibertyAttributeMap)
-
 def metadata_validator(meta):
     provider=lasso.Provider.newFromBuffer(lasso.PROVIDER_ROLE_ANY, meta.encode('utf8'))
     if not provider:
@@ -85,59 +76,6 @@ def organization_name(provider):
     else:
         return provider.providerId
 
-class LibertyProvider(models.Model):
-    entity_id = models.URLField(unique = True)
-    entity_id_sha1 = models.CharField(max_length = 40, blank=True)
-    name = models.CharField(max_length = 40, unique = True,
-            help_text = _("Internal nickname for the service provider"),
-            blank = True)
-    protocol_conformance = models.IntegerField(max_length = 10,
-            choices = ((0, 'SAML 1.0'),
-                       (1, 'SAML 1.1'),
-                       (2, 'SAML 1.2'),
-                       (3, 'SAML 2.0')))
-    metadata = models.TextField(validators = [ metadata_validator ])
-    # All following field must be PEM formatted textual data
-    public_key = models.TextField(blank=True)
-    ssl_certificate = models.TextField(blank=True)
-    ca_cert_chain = models.TextField(blank=True)
-
-    def __unicode__(self):
-        return self.name
-
-    def save(self, *args, **kwargs):
-        '''Update the SHA1 hash of the entity_id when saving'''
-        if self.protocol_conformance == 3:
-            self.entity_id_sha1 = hashlib.sha1(self.entity_id).hexdigest()
-        super(LibertyProvider, self).save(*args, **kwargs)
-
-    @classmethod
-    def get_provider_by_samlv2_artifact(cls, artifact):
-        '''Find a provider whose SHA-1 hash of its entityID is the 5-th to the
-           25-th byte of the given artifact'''
-        try:
-            artifact = base64.b64decode(artifact)
-        except:
-            raise ValueError('Artifact is not a base64 encoded value')
-        entity_id_sha1 = artifact[4:24]
-        entity_id_sha1 = binascii.hexlify(entity_id_sha1)
-        try:
-            return cls.objects.get(entity_id_sha1=entity_id_sha1)
-        except cls.DoesNotExist:
-            return None
-
-    def clean(self):
-        super(LibertyProvider, self).clean()
-        if self.metadata:
-            p = lasso.Provider.newFromBuffer(lasso.PROVIDER_ROLE_ANY, self.metadata.encode('utf8'))
-            if p:
-                self.entity_id = p.providerId
-                if not self.name:
-                    self.name = organization_name(p)
-                self.protocol_conformance = p.protocolConformance
-        else:
-            print 'coin'
-
 # TODO: Remove this in LibertyServiceProvider
 ASSERTION_CONSUMER_PROFILES = (
         ('meta', _('Use the default from the metadata file')),
@@ -180,6 +118,176 @@ def nidformat_to_saml2_urn(key):
 def nidformat_to_idff12_urn(key):
     return NAME_ID_FORMATS.get(key, {}).get('idff12')
 
+# According to: saml-profiles-2.0-os
+# The HTTP Redirect binding MUST NOT be used, as the response will typically exceed the URL length permitted by most user agents.
+BINDING_SSO_IDP = (
+    (lasso.SAML2_METADATA_BINDING_ARTIFACT, _('Artifact binding')),
+    (lasso.SAML2_METADATA_BINDING_POST, _('POST binding'))
+)
+HTTP_METHOD = (
+    (lasso.HTTP_METHOD_REDIRECT, _('Redirect binding')),
+    (lasso.HTTP_METHOD_SOAP, _('SOAP binding'))
+)
+
+USER_CONSENT = (
+    ('urn:oasis:names:tc:SAML:2.0:consent:unspecified', _('Unspecified')),
+    ('urn:oasis:names:tc:SAML:2.0:consent:obtained', _('Obtained')),
+    ('urn:oasis:names:tc:SAML:2.0:consent:prior', _('Prior')),
+    ('urn:oasis:names:tc:SAML:2.0:consent:current-explicit', _('Explicit')),
+    ('urn:oasis:names:tc:SAML:2.0:consent:current-implicit', _('Implicit')),
+    ('urn:oasis:names:tc:SAML:2.0:consent:unavailable', _('Unavailable')),
+    ('urn:oasis:names:tc:SAML:2.0:consent:inapplicable', _('Inapplicable'))
+)
+
+class LibertyAttributeMap(models.Model):
+    name = models.CharField(max_length = 40, unique = True)
+    def __unicode__(self):
+        return self.name
+
+class LibertyAttributeMapping(models.Model):
+    source_attribute_name = models.CharField(max_length = 40)
+    attribute_value_format = models.URLField(choices = ATTRIBUTE_VALUE_FORMATS)
+    attribute_name = models.CharField(max_length = 40)
+    map = models.ForeignKey(LibertyAttributeMap)
+
+class IdPOptionsSPPolicy(models.Model):
+    name = models.CharField(_('name'), max_length=80, unique=True)
+    enabled = models.BooleanField(verbose_name = _('Enabled'))
+    no_nameid_policy = models.BooleanField(
+            verbose_name = _("Do not send a nameId Policy"))
+    requested_name_id_format = models.CharField(max_length = 20,
+            default = DEFAULT_NAME_ID_FORMAT,
+            choices = NAME_ID_FORMATS_CHOICES)
+    transient_is_persistent = models.BooleanField(
+            verbose_name = \
+            _("This IdP falsely sends a transient NameID \
+            which is in fact persistent"))
+    allow_create = models.BooleanField(
+            verbose_name = _("Allow IdP to create an identity"))
+    enable_binding_for_sso_response = models.BooleanField(
+            verbose_name = _('Binding for Authnresponse \
+            (taken from metadata by the IdP if not enabled)'))
+    binding_for_sso_response = models.CharField(
+            max_length = 60, choices = BINDING_SSO_IDP,
+            verbose_name = '',
+            default = lasso.SAML2_METADATA_BINDING_ARTIFACT)
+    enable_http_method_for_slo_request = models.BooleanField(
+            verbose_name = _('HTTP method for single logout request \
+            (taken from metadata if not enabled)'))
+    http_method_for_slo_request = models.IntegerField(
+            max_length = 60, choices = HTTP_METHOD,
+            verbose_name = '',
+            default = lasso.HTTP_METHOD_REDIRECT)
+    enable_http_method_for_defederation_request = models.BooleanField(
+            verbose_name = \
+            _('HTTP method for federation termination request \
+            (taken from metadata if not enabled)'))
+    http_method_for_defederation_request = models.IntegerField(
+            max_length = 60, choices = HTTP_METHOD,
+            verbose_name = '',
+            default = lasso.HTTP_METHOD_SOAP)
+    user_consent = models.CharField(
+            max_length = 60, choices = USER_CONSENT,
+            verbose_name = _("Ask user consent"),
+            default = 'urn:oasis:names:tc:SAML:2.0:consent:current-implicit')
+    want_force_authn_request = models.BooleanField(
+            verbose_name = _("Force authentication"))
+    want_is_passive_authn_request = models.BooleanField(
+            verbose_name = _("Passive authentication"))
+    want_authn_request_signed = models.BooleanField(
+            verbose_name = _("Want AuthnRequest signed"))
+
+    class Meta:
+        verbose_name = _('identity provider options policy')
+        verbose_name_plural = _('identity provider options policies')
+
+    def __unicode__(self):
+        return self.name
+
+class AuthorizationAttributeMap(models.Model):
+    name = models.CharField(max_length = 40, unique = True)
+    def __unicode__(self):
+        return self.name
+
+class AuthorizationAttributeMapping(models.Model):
+    source_attribute_name = models.CharField(max_length = 40,
+            blank=True)
+    attribute_value_format = models.CharField(max_length = 40,
+            blank=True)
+    attribute_name = models.CharField(max_length = 40)
+    attribute_value = models.CharField(max_length = 40)
+    map = models.ForeignKey(AuthorizationAttributeMap)
+
+class AuthorizationSPPolicy(models.Model):
+    name = models.CharField(_('name'), max_length=80, unique=True)
+    enabled = models.BooleanField(verbose_name = _('Enabled'))
+    attribute_map = models.ForeignKey(AuthorizationAttributeMap,
+            related_name = "authorization_attributes",
+            blank = True, null = True)
+    default_denial_message = models.CharField(
+            max_length = 80,
+            verbose_name = \
+            _("Default message to display to the user when access is denied"),
+            default=_('You are not authorized to access the service.'))
+
+    class Meta:
+        verbose_name = _('authorization policy')
+        verbose_name_plural = _('authorization policies')
+
+    def __unicode__(self):
+        return self.name
+
+class LibertyProvider(models.Model):
+    entity_id = models.URLField(unique = True)
+    entity_id_sha1 = models.CharField(max_length = 40, blank=True)
+    name = models.CharField(max_length = 140,
+            help_text = _("Internal nickname for the service provider"),
+            blank = True)
+    protocol_conformance = models.IntegerField(max_length = 10,
+            choices = ((0, 'SAML 1.0'),
+                       (1, 'SAML 1.1'),
+                       (2, 'SAML 1.2'),
+                       (3, 'SAML 2.0')))
+    metadata = models.TextField(validators = [ metadata_validator ])
+    # All following field must be PEM formatted textual data
+    public_key = models.TextField(blank=True)
+    ssl_certificate = models.TextField(blank=True)
+    ca_cert_chain = models.TextField(blank=True)
+    federation_source = models.CharField(max_length=64, blank=True, null=True)
+
+    def __unicode__(self):
+        return self.name
+
+    def save(self, *args, **kwargs):
+        '''Update the SHA1 hash of the entity_id when saving'''
+        if self.protocol_conformance == 3:
+            self.entity_id_sha1 = hashlib.sha1(self.entity_id).hexdigest()
+        super(LibertyProvider, self).save(*args, **kwargs)
+
+    @classmethod
+    def get_provider_by_samlv2_artifact(cls, artifact):
+        '''Find a provider whose SHA-1 hash of its entityID is the 5-th to the
+           25-th byte of the given artifact'''
+        try:
+            artifact = base64.b64decode(artifact)
+        except:
+            raise ValueError('Artifact is not a base64 encoded value')
+        entity_id_sha1 = artifact[4:24]
+        entity_id_sha1 = binascii.hexlify(entity_id_sha1)
+        try:
+            return cls.objects.get(entity_id_sha1=entity_id_sha1)
+        except cls.DoesNotExist:
+            return None
+
+    def clean(self):
+        super(LibertyProvider, self).clean()
+        p = lasso.Provider.newFromBuffer(lasso.PROVIDER_ROLE_ANY, self.metadata.encode('utf8'))
+        self.entity_id = p.providerId
+        if not self.name:
+            self.name = organization_name(p)
+        if not self.protocol_conformance:
+            self.protocol_conformance = p.protocolConformance
+
 # TODO: The IdP must look to the preferred binding order for sso in the SP metadata (AssertionConsumerService)
 # expect if the protocol for response is defined in the request (ProtocolBinding attribute)
 class LibertyServiceProvider(models.Model):
@@ -215,27 +323,6 @@ class LibertyServiceProvider(models.Model):
     ask_user_consent = models.BooleanField(
         verbose_name = _('Ask user for consent when creating a federation'), default = False)
 
-# According to: saml-profiles-2.0-os
-# The HTTP Redirect binding MUST NOT be used, as the response will typically exceed the URL length permitted by most user agents.
-BINDING_SSO_IDP = (
-    (lasso.SAML2_METADATA_BINDING_ARTIFACT, _('Artifact binding')),
-    (lasso.SAML2_METADATA_BINDING_POST, _('POST binding'))
-)
-HTTP_METHOD = (
-    (lasso.HTTP_METHOD_REDIRECT, _('Redirect binding')),
-    (lasso.HTTP_METHOD_SOAP, _('SOAP binding'))
-)
-
-USER_CONSENT = (
-    ('urn:oasis:names:tc:SAML:2.0:consent:unspecified', _('Unspecified')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:obtained', _('Obtained')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:prior', _('Prior')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:current-explicit', _('Explicit')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:current-implicit', _('Implicit')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:unavailable', _('Unavailable')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:inapplicable', _('Inapplicable'))
-)
-
 # TODO: The choice for requests must be restricted by the IdP metadata
 # The SP then chooses the binding in this list.
 # For response, if the requester uses a (a)synchronous binding, the responder uses the same.
@@ -244,47 +331,13 @@ class LibertyIdentityProvider(models.Model):
     liberty_provider = models.OneToOneField(LibertyProvider,
             primary_key = True, related_name = 'identity_provider')
     enabled = models.BooleanField(verbose_name = _('Enabled'))
-    enable_following_policy = models.BooleanField(verbose_name = \
-        _('The following options policy will apply except if the policy for all identity provider is defined.'))
-    no_nameid_policy = models.BooleanField(
-            verbose_name = _("Do not send a nameId Policy"))
-    requested_name_id_format = models.CharField(max_length = 20,
-            default = DEFAULT_NAME_ID_FORMAT,
-            choices = NAME_ID_FORMATS_CHOICES)
-    allow_create = models.BooleanField(
-            verbose_name = _("Allow IdP to create an identity"))
-    enable_binding_for_sso_response = models.BooleanField(
-            verbose_name = _('Binding for Authnresponse (taken from metadata by the IdP if not enabled)'))
-    binding_for_sso_response = models.CharField(
-            max_length = 60, choices = BINDING_SSO_IDP,
-            verbose_name = '',
-            default = lasso.SAML2_METADATA_BINDING_ARTIFACT)
-    enable_http_method_for_slo_request = models.BooleanField(
-            verbose_name = _('HTTP method for single logout request (taken from metadata if not enabled)'))
-    http_method_for_slo_request = models.IntegerField(
-            max_length = 60, choices = HTTP_METHOD,
-            verbose_name = '',
-            default = lasso.HTTP_METHOD_REDIRECT)
-    enable_http_method_for_defederation_request = models.BooleanField(
-            verbose_name = _('HTTP method for federation termination request (taken from metadata if not enabled)'))
-    http_method_for_defederation_request = models.IntegerField(
-            max_length = 60, choices = HTTP_METHOD,
-            verbose_name = '',
-            default = lasso.HTTP_METHOD_SOAP)
-    user_consent = models.CharField(
-            max_length = 60, choices = USER_CONSENT,
-            verbose_name = _("Ask user consent"),
-            default = 'urn:oasis:names:tc:SAML:2.0:consent:current-implicit')
-    want_force_authn_request = models.BooleanField(
-            verbose_name = _("Force authentication"))
-    want_is_passive_authn_request = models.BooleanField(
-            verbose_name = _("Passive authentication"))
-    want_authn_request_signed = models.BooleanField(
-            verbose_name = _("Want AuthnRequest signed"))
-    # Mapping to use to get User attributes from the assertion
-    attribute_map = models.ForeignKey(LibertyAttributeMap,
-            related_name = "identity_providers",
-            blank = True, null = True)
+    enable_following_idp_options_policy = models.BooleanField(verbose_name = \
+        _('The following options policy will apply except if a policy for all identity provider is defined.'))
+    idp_options_policy = models.ForeignKey(IdPOptionsSPPolicy, related_name = "idp_options_policy", verbose_name = _('IdP Options Policy'), blank=True, null=True)
+    enable_following_authorization_policy = models.BooleanField(verbose_name = \
+        _('The following authorization policy will apply except if a policy for all identity provider is defined.'))
+    authorization_policy = models.ForeignKey(AuthorizationSPPolicy, related_name = "authorization_policy", verbose_name = _('Authorization Policy'), blank=True, null=True)
+
     # TODO: add clean method which checks that the LassoProvider we can create
     # with the metadata file support the IDP role
     # i.e. provider.roles & lasso.PROVIDER_ROLE_IDP != 0
@@ -299,6 +352,25 @@ class LibertyIdentityDump(models.Model):
     user = models.ForeignKey(User, unique = True)
     identity_dump = models.TextField(blank = True)
 
+class SessionLinkedManager(models.Manager):
+    def cleanup(self):
+        engine = import_module(settings.SESSION_ENGINE)
+        store = engine.SessionStore()
+        for o in self.all():
+            key = o.django_session_key
+            if not store.exists(key):
+                o.delete()
+            else:
+                session = engine.SessionStore(session_key=key)
+                if session.get_expiry_date() >= datetime.datetime.now():
+                    store.delete(key)
+                    o.delete()
+
+LIBERTY_SESSION_DUMP_KIND_SP = 0
+LIBERTY_SESSION_DUMP_KIND_IDP = 1
+LIBERTY_SESSION_DUMP_KIND = { LIBERTY_SESSION_DUMP_KIND_SP: 'sp',
+        LIBERTY_SESSION_DUMP_KIND_IDP: 'idp' }
+
 class LibertySessionDump(models.Model):
     '''Store lasso session object dump.
 
@@ -306,6 +378,9 @@ class LibertySessionDump(models.Model):
        assertions through the LibertySession object'''
     django_session_key = models.CharField(max_length = 40)
     session_dump = models.TextField(blank = True)
+    kind = models.IntegerField(choices = LIBERTY_SESSION_DUMP_KIND.items())
+
+    objects = SessionLinkedManager()
 
 class LibertyManageDump(models.Model):
     '''Store lasso manage dump
@@ -315,6 +390,15 @@ class LibertyManageDump(models.Model):
     django_session_key = models.CharField(max_length = 40)
     manage_dump = models.TextField(blank = True)
 
+    objects = SessionLinkedManager()
+
+class LibertyArtifactManager(models.Manager):
+    def cleanup(self):
+        # keep artifacts 10 minutes
+        expire = getattr(settings, 'SAML2_ARTIFACT_EXPIRATION', 600)
+        before = datetime.datetime.now()-datetime.timedelta(seconds=expire)
+        self.filter(creation__lt=before).delete()
+
 class LibertyArtifact(models.Model):
     """Store an artifact and the associated XML content"""
     creation = models.DateTimeField(auto_now_add=True)
@@ -323,12 +407,21 @@ class LibertyArtifact(models.Model):
     django_session_key = models.CharField(max_length = 40)
     provider_id = models.CharField(max_length = 80)
 
+    objects = LibertyArtifactManager()
+
 def nameid2kwargs(name_id):
     return { 'name_id_qualifier': name_id.nameQualifier,
         'name_id_sp_name_qualifier': name_id.spNameQualifier,
         'name_id_content': name_id.content,
         'name_id_format': name_id.format }
 
+class LibertyAssertionManager(models.Manager):
+    def cleanup(self):
+        # keep assertions 1 week
+        expire = getattr(settings, 'SAML2_ASSERTION_EXPIRATION', 3600*24*7)
+        before = datetime.datetime.now()-datetime.timedelta(seconds=expire)
+        self.filter(creation__lt=before).delete()
+
 class LibertyAssertion(models.Model):
     assertion_id = models.CharField(max_length = 50)
     provider_id = models.CharField(max_length = 80)
@@ -352,15 +445,15 @@ class LibertyFederation(models.Model):
     idp_id = models.CharField(max_length=80)
     sp_id = models.CharField(max_length=80)
     name_id_qualifier = models.CharField(max_length = 150,
-            verbose_name = "NameQualifier")
+            verbose_name = "NameQualifier", blank=True, null=True)
     name_id_format = models.CharField(max_length = 100,
-            verbose_name = "NameIDFormat")
+            verbose_name = "NameIDFormat", blank=True, null=True)
     name_id_content = models.CharField(max_length = 100,
             verbose_name = "NameID")
     name_id_sp_name_qualifier = models.CharField(max_length = 100,
-            verbose_name = "SPNameQualifier")
+            verbose_name = "SPNameQualifier", blank=True, null=True)
     name_id_sp_provided_id = models.CharField(max_length=100,
-            verbose_name="SPProvidedID")
+            verbose_name="SPProvidedID", blank=True, null=True)
 
     def __init__(self, *args, **kwargs):
         saml2_assertion = kwargs.pop('saml2_assertion', None)
@@ -394,6 +487,8 @@ class LibertySession(models.Model):
             verbose_name = _("SPNameQualifier"), null = True)
     creation = models.DateTimeField(auto_now_add=True)
 
+    objects = SessionLinkedManager()
+
     def __init__(self, *args, **kwargs):
         saml2_assertion = kwargs.pop('saml2_assertion', None)
         if saml2_assertion:
