@@ -5,7 +5,7 @@ import os
 gettext_noop = lambda s: s
 
 DEBUG = True
-USE_DEBUG_TOOLBAR = True
+USE_DEBUG_TOOLBAR = False
 STATIC_SERVE = True
 TEMPLATE_DEBUG = DEBUG
 _PROJECT_PATH = os.path.join(os.path.dirname(__file__), '..')
@@ -16,17 +16,16 @@ ADMINS = (
 
 MANAGERS = ADMINS
 
-_DATABASE_NAME = os.path.join(_PROJECT_PATH, 'authentic.db')
 DATABASES = {
     'default': {
         'ENGINE': 'django.db.backends.sqlite3',
-        'NAME': _DATABASE_NAME,
+        'NAME': os.path.join(_PROJECT_PATH, 'authentic2/authentic.db') 
     }
 }
 
 #session cookie parameters
 SESSION_EXPIRE_AT_BROWSER_CLOSE =  True
-SESSION_COOKIE_AGE = 36000
+SESSION_COOKIE_AGE = 36000 # one day of work
 
 # Local time zone for this installation. Choices can be found here:
 # http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
@@ -97,14 +96,14 @@ MIDDLEWARE_CLASSES = (
     'django.middleware.csrf.CsrfViewMiddleware',
     'django.contrib.auth.middleware.AuthenticationMiddleware',
     'django.contrib.messages.middleware.MessageMiddleware',
-    'authentic2.admin_log_view.middleware.LoggerMiddleware',
     'authentic2.idp.middleware.DebugMiddleware'
 )
 
 ROOT_URLCONF = 'authentic2.urls'
 
 TEMPLATE_DIRS = (
-    os.path.join(_PROJECT_PATH, 'templates'),
+    # authentic2/templates
+    os.path.join(os.path.dirname(__file__), 'templates'),
 )
 
 INSTALLED_APPS = (
@@ -118,11 +117,11 @@ INSTALLED_APPS = (
     'authentic2.idp',
     'authentic2.idp.saml',
     'registration',
-    'authentic2.sslauth',
-    'authentic2.admin_log_view',
-    'authentic2.auth',
-    'authentic2.auth.openid',
-    'authentic2.auth.oath',
+    'authentic2.auth2_auth',
+    'authentic2.auth2_auth.auth2_openid',
+    'authentic2.auth2_auth.auth2_oath',
+    'authentic2.auth2_auth.auth2_ssl',
+    'south',
 )
 
 MESSAGE_STORAGE = 'django.contrib.messages.storage.session.SessionStorage'
@@ -132,22 +131,54 @@ LOGIN_URL = '/login'
 # Registration settings
 ACCOUNT_ACTIVATION_DAYS = 2
 EMAIL_HOST = 'localhost'
-DEFAULT_FROM_EMAIL = 'webmaster@entrouvert.com'
+DEFAULT_FROM_EMAIL = 'webmaster@localhost'
 LOGIN_REDIRECT_URL = '/'
-AUTHENTIC2_USER_IFRAME = False
+# Default profile class
+AUTH_PROFILE_MODULE = 'idp.UserProfile'
 
 INTERNAL_IPS = ('127.0.0.1',)
 DEBUG_TOOLBAR_CONFIG = {'INTERCEPT_REDIRECTS': False}
 
-# SAML settings
+###########################
+# Authentication settings
+###########################
+
 # Only RSA private keys are currently supported
-AUTH_FRONTENDS = ( 'authentic2.auth.backend.LoginPasswordBackend',
-        'authentic2.auth.openid.backend.OpenIDFrontend',
-        'authentic2.auth.backend.SSLFrontend',
-        'authentic2.authsaml2.frontend.AuthSAML2Frontend',
-        'authentic2.auth.oath.frontend.OATHOTPFrontend')
+AUTH_FRONTENDS = ( 'authentic2.auth2_auth.backend.LoginPasswordBackend',)
+
+AUTHENTICATION_BACKENDS = (
+    'django.contrib.auth.backends.ModelBackend',
+)
+
+# expiration in seconds of authentication events.
+# default: 1 week
+# AUTHENTICATION_EVENT_EXPIRATION = 3600*24*7
+
+# SSL Authentication
+AUTH_SSL = True
+SSLAUTH_CREATE_USER = True
+
+# SAML2 Authentication
+AUTH_SAML2 = True
+
+# OpenID Authentication
+AUTH_OPENID = True
+
+# OATH Authentication
+AUTH_OATH = True
+
+#############################
+# Identity Provider settings
+#############################
+
+# List of IdP backends, mainly used to show available services in the homepage
+# of user, and to handle SLO for each protocols
+IDP_BACKENDS = [ ]
+
+# SAML2 IDP
 IDP_SAML2 = True
-IDP_IDFF12 = True
+
+# You MUST changes these keys, they are just for testing !
 SAML_SIGNING_KEY = '''-----BEGIN CERTIFICATE-----
 MIIDIzCCAgugAwIBAgIJANUBoick1pDpMA0GCSqGSIb3DQEBBQUAMBUxEzARBgNV
 BAoTCkVudHJvdXZlcnQwHhcNMTAxMjE0MTUzMzAyWhcNMTEwMTEzMTUzMzAyWjAV
@@ -167,6 +198,7 @@ lG6l41SXp6YgIb2ToT+rOKdIGIQuGDlzeR88fDxWEU0vEujZv/v1PE1YOV0xKjTT
 JumlBc6IViKhJeo1wiBBrVRIIkKKevHKQzteK8pWm9CYWculxT26TZ4VWzGbo06j
 o2zbumirrLLqnt1gmBDvDvlOwC/zAAyL4chbz66eQHTiIYZZvYgy
 -----END CERTIFICATE-----'''
+
 SAML_PRIVATE_KEY = '''-----BEGIN RSA PRIVATE KEY-----
 MIIEpAIBAAKCAQEAvxFkfPdndlGgQPDZgFGXbrNAc/79PULZBuNdWFHDD9P5hNhZ
 n9Kqm4Cp06Pe/A6u+g5wLnYvbZQcFCgfQAEzziJtb3J55OOlB7iMEI/T2AX2WzrU
@@ -194,39 +226,28 @@ gmsgaiMCgYB/nrTk89Fp7050VKCNnIt1mHAcO9cBwDV8qrJ5O3rIVmrg1T6vn0aY
 wRiVcNacaP+BivkrMjr4BlsUM6yH4MOBsNhLURiiCL+tLJV7U0DWlCse/doWij4U
 TKX6tp6oI+7MIJE6ySZ0cBqOiydAkBePZhu57j6ToBkTa0dbHjn1WA==
 -----END RSA PRIVATE KEY-----'''
+
 SAML_METADATA_ROOT = 'metadata'
+# Whether to autoload SAML 2.0 identity providers and services metadata
+# Only https URLS are accepted.
 # Can be none, sp, idp or both
 SAML_METADATA_AUTOLOAD = 'none'
 
-# SSL settings
-AUTH_SSL = True
-SSLAUTH_CREATE_USER = True
-AUTHENTICATION_BACKENDS = (
-    'django.contrib.auth.backends.ModelBackend',
-    'authentic2.sslauth.backends.SSLAuthBackend',
-    'authentic2.auth.oath.backend.OATHTOTPBackend'
-)
-
-# IDP
-IDP_BACKENDS = [ 'authentic2.idp.saml.backend.SamlBackend',
-        'authentic2.idp.AdminBackend' ]
-
-#AuthSAML2 Configuration
-INSTALLED_APPS += ('authentic2.authsaml2',)
-AUTHENTICATION_BACKENDS += (
-        'authentic2.authsaml2.backends.AuthSAML2PersistentBackend',
-        'authentic2.authsaml2.backends.AuthSAML2TransientBackend')
-
 # OpenID settings
-AUTH_OPENID = True
-IDP_OPENID = False
-IDPOI_PATH = '/o8/'
+IDP_OPENID = True
 
+# CAS settings
+IDP_CAS = False
+# expiration time in seconds of the cas tickets
+# CAS_TICKET_EXPIRATION = 240
+
+import logging
 # Logging settings
 LOG_FILENAME = os.path.join(_PROJECT_PATH, 'log.log')
-LOG_FILE_LEVEL = 10 #CRITICAL 50 ERROR 40 WARNING 30 INFO 20 DEBUG 10
+LOG_FILE_LEVEL = logging.DEBUG
 LOG_SYSLOG = True
-LOG_SYS_LEVEL = 10
+LOG_SYS_LEVEL = logging.WARNING
+
 
 # local_settings.py can be used to override environment-specific settings
 # like database and email that differ between development and production.
@@ -239,9 +260,65 @@ if USE_DEBUG_TOOLBAR:
     MIDDLEWARE_CLASSES += ('debug_toolbar.middleware.DebugToolbarMiddleware',)
     INSTALLED_APPS += ('debug_toolbar',)
 
+if AUTH_SAML2:
+    INSTALLED_APPS += ('authentic2.authsaml2',)
+    AUTHENTICATION_BACKENDS += (
+            'authentic2.authsaml2.backends.AuthSAML2PersistentBackend',
+            'authentic2.authsaml2.backends.AuthSAML2TransientBackend')
+    AUTH_FRONTENDS += ('authentic2.authsaml2.frontend.AuthSAML2Frontend',)
+    IDP_BACKENDS += ('authentic2.authsaml2.backends.AuthSAML2Backend',)
+
 if AUTH_OPENID:
     INSTALLED_APPS += ('django_authopenid',)
+    AUTH_FRONTENDS += ('authentic2.auth2_auth.auth2_openid.backend.OpenIDFrontend',)
+
+if AUTH_SSL:
+    AUTHENTICATION_BACKENDS += ('authentic2.auth2_auth.auth2_ssl.backend.SSLBackend',)
+    AUTH_FRONTENDS += ('authentic2.auth2_auth.auth2_ssl.frontend.SSLFrontend',)
+
+if AUTH_OATH:
+    AUTHENTICATION_BACKENDS += ('authentic2.auth2_auth.auth2_oath.backend.OATHTOTPBackend',)
+    AUTH_FRONTENDS += ('authentic2.auth2_auth.auth2_oath.frontend.OATHOTPFrontend',)
+
+if IDP_SAML2:
+    IDP_BACKENDS += ('authentic2.idp.saml.backend.SamlBackend',)
 
 if IDP_OPENID:
-    INSTALLED_APPS += ('authentic2.django_openid_provider',
-                    'openid_provider',)
+    INSTALLED_APPS += ('authentic2.idp.idp_openid',)
+
+if IDP_CAS:
+    INSTALLED_APPS += ('authentic2.idp.idp_cas',)
+
+from logging.handlers import SysLogHandler
+import threading
+import sys
+
+_LOCAL = threading.local()
+
+def getlogger():
+
+    logger = getattr(_LOCAL, 'logger', None)
+    if logger is not None:
+        return logger
+    logger = logging.getLogger()
+    logger.setLevel(logging.DEBUG)
+
+    formatter = logging.Formatter('[%(asctime)s] %(levelname)-8s %(name)s.%(message)s','%Y-%m-%d %a %H:%M:%S')
+
+    if LOG_FILENAME:
+        log_handler = logging.FileHandler(LOG_FILENAME)
+        log_handler.setFormatter(formatter)
+        log_handler.setLevel(LOG_FILE_LEVEL)
+        logger.addHandler(log_handler)
+
+    if LOG_SYSLOG:
+        syslog_handler = SysLogHandler(address = '/dev/log')
+        formatter = logging.Formatter('authentic %(levelname)-8s %(name)s.%(message)s','%Y-%m-%d %a %H:%M:%S')
+        syslog_handler.setFormatter(formatter)
+        syslog_handler.setLevel(LOG_SYS_LEVEL)
+        logger.addHandler(syslog_handler)
+
+    setattr(_LOCAL,'logger',logger)
+    return logger
+
+getlogger()
